@@ -31,6 +31,7 @@
 #include <pthread.h>
 #include "levelbar.h"
 #include "portlib.h"
+#include "piano.h" // From swami.
 
 #define KEY_UNKNOWN	(-1)
 #define MIDI_CHANNELS	16
@@ -75,6 +76,7 @@ struct channel_status_t {
 	GtkWidget *w_chnum, *w_prog;
 	GtkWidget *w_vel, *w_main, *w_exp;
 	GtkWidget *w_pan, *w_pitch;
+	GtkWidget *w_piano;
 };
 
 enum {
@@ -116,10 +118,14 @@ enum {
 	V_EXP,
 	V_PAN,
 	V_PITCH,
-	/*V_PIANO,*/
+	V_PIANO,
 	V_COLS
 };
 
+static int show_piano = TRUE;
+static int aseqview_cols = V_COLS;
+
+
 /*
  * prototypes
  */
@@ -138,6 +144,7 @@ static GtkWidget *create_viewer(port_status_t *port);
 static void create_viewer_titles(GtkWidget *table);
 static void set_vel_bar_color(GtkWidget *w, int is_drum, int in_buf);
 static void av_channel_update(GtkWidget *w, int val, int in_buf);
+static void av_note_update(GtkWidget *w, int key, int note_on, int in_buf);
 static void create_channel_viewer(GtkWidget *table, port_status_t *st, int ch);
 static void mute_channel(GtkToggleButton *w, channel_status_t *chst);
 static void send_notes_off(channel_status_t *chst);
@@ -190,6 +197,7 @@ static struct option long_option[] = {
 	{"help", 0, NULL, 'h'},
 	{"thread", 0, NULL, 't'},
 	{"nothread", 0, NULL, 'm'},
+	{"nopiano", 0, NULL, 'P'},
 	{NULL, 0, NULL, 0},
 };
 
@@ -203,7 +211,7 @@ int main(int argc, char **argv)
 
 	gtk_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "ors:d:p:tm", long_option, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "ors:d:p:tmP", long_option, NULL)) != -1) {
 		switch (c) {
 		case 'o':
 			do_output = FALSE;
@@ -232,6 +240,10 @@ int main(int argc, char **argv)
 		case 'm':
 			use_thread = FALSE;
 			break;
+		case 'P':
+			show_piano = FALSE;
+			aseqview_cols = V_COLS - 1;
+			break;
 		default:
 			usage();
 			exit(1);
@@ -367,6 +379,8 @@ static int av_ringbuf_read(int *type, GtkWidget **w, long *data)
 enum {
 	VEL_COLOR,
 	UPDATE_STATUS,
+	NOTE_ON,
+	NOTE_OFF,
 	UPDATE_PGM,
 	UPDATE_MODE
 };
@@ -405,6 +419,12 @@ idle_cb(gpointer data)
 		case UPDATE_MODE:
 			expose_midi_mode(w);
 			break;
+		case NOTE_ON:
+			av_note_update(w, val, 1, 0);
+			break;
+		case NOTE_OFF:
+			av_note_update(w, val, 0, 0);
+			break;
 		}
 	}
 	usleep(1000);
@@ -551,7 +571,7 @@ usage(void)
 {
 	printf("%s - ALSA sequencer event viewer / filter\n", PACKAGE);
 	printf("  ver.%s\n", VERSION);
-	printf("  Copyright (c) 1999-2000 by Takashi Iwai <iwai@ww.uni-erlangen.de>\n");
+	printf("  Copyright (c) 1999-2005 by Takashi Iwai <tiwai@suse.de>\n");
 	printf("\n");
 		
 	printf("usage: %s [-options]\n", PACKAGE);
@@ -562,6 +582,7 @@ usage(void)
 	printf("   -d,--dest addr    output to specified addr (client:port)\n");
 	printf("   -t,--thread       use multi-threads (default)\n");
 	printf("   -m,--nothread     don't use multi-threads\n");
+	printf("   -P,--nopiano      don't show piano\n");
 }
 
 
@@ -736,8 +757,8 @@ create_viewer(port_status_t *port)
 	GtkWidget *table;
 	int i;
 
-	table = gtk_table_new(V_COLS, MIDI_CHANNELS + 1, FALSE);
-	for (i = 0; i < V_COLS - 1; i++)
+	table = gtk_table_new(aseqview_cols, MIDI_CHANNELS + 1, FALSE);
+	for (i = 0; i < aseqview_cols - 1; i++)
 		gtk_table_set_col_spacing(GTK_TABLE(table), i, 4);
 	create_viewer_titles(table);
 	for (i = 0; i < MIDI_CHANNELS; i++)
@@ -783,6 +804,12 @@ create_viewer_titles(GtkWidget *table)
 	w = gtk_label_new("Pitch");
 	gtk_table_attach_defaults(GTK_TABLE(table), w, V_PITCH, V_PITCH+1, 0, 1);
 	gtk_widget_show(w);
+
+	if (show_piano) {
+		w = gtk_label_new("Piano");
+		gtk_table_attach_defaults(GTK_TABLE(table), w, V_PIANO, V_PIANO+1, 0, 1);
+		gtk_widget_show(w);
+	}
 }
 
 
@@ -875,12 +902,14 @@ create_channel_viewer(GtkWidget *table, port_status_t *st, int ch)
 	chst->w_pitch = w;
 
 	/* piano */
-	/*
-	w = create_piano;
-I	gtk_table_attach_defaults(GTK_TABLE(table), w, V_PIANO, V_PIANO + 1,
-	top, bottom);
-	gtk_widget_show(w);
-	*/
+	if (show_piano) {
+		w = piano_new(NULL);
+		gtk_table_attach_defaults(GTK_TABLE(table), w, V_PIANO, V_PIANO + 1,
+					  top, bottom);
+		chst->w_piano = w;
+		gtk_widget_show(w);
+	}
+	
 }
 
 
@@ -909,6 +938,7 @@ static void
 send_notes_off(channel_status_t *chst)
 {
 	snd_seq_event_t tmpev;
+	int i;
 
 	snd_seq_ev_clear(&tmpev);
 	snd_seq_ev_set_direct(&tmpev);
@@ -925,6 +955,7 @@ static void
 send_resets(channel_status_t *chst)
 {
 	snd_seq_event_t tmpev;
+	int i;
 
 	snd_seq_ev_clear(&tmpev);
 	snd_seq_ev_set_direct(&tmpev);
@@ -1051,6 +1082,24 @@ av_channel_update(GtkWidget *w, int val, int in_buf)
 		channel_status_bar_update(w, val);
 }
 
+/*
+ */
+static void
+av_note_update(GtkWidget *w, int key, int note_on, int in_buf)
+{
+	if (in_buf) {
+		if (note_on)
+			av_ringbuf_write(NOTE_ON, w, key);
+		else
+			av_ringbuf_write(NOTE_OFF, w, key);
+	} else {
+		if (note_on)
+			piano_note_on(PIANO(w), key);
+		else
+			piano_note_off(PIANO(w), key);
+	}
+}
+
 /*
  * redirect an event to subscribers port
  */
@@ -1172,6 +1221,12 @@ change_note(port_status_t *port, int ch, int key, int vel, int in_buf)
 			av_channel_update(chst->w_vel, chst->max_vel, in_buf);
 		}
 	}
+	if (show_piano) {
+		if (chst->vel[key] > 0)
+			av_note_update(chst->w_piano, key, 1, in_buf);
+		else
+			av_note_update(chst->w_piano, key, 0, in_buf);
+	}
 }
 
 /*
@@ -1298,7 +1353,7 @@ reset_controllers(channel_status_t *chst, int in_buf)
 static void
 reset_all(midi_status_t *st)
 {
-	int i, p;
+	int i, p, j;
 
 	st->midi_mode = MIDI_MODE_GM;
 	st->timer_update = TRUE;
@@ -1320,6 +1375,10 @@ reset_all(midi_status_t *st)
 			else
 				chst->is_drum = 0;
 			set_vel_bar_color(chst->w_vel, chst->is_drum, 0);
+			if (chst->w_piano) {
+				for (j = 0; j < NUM_KEYS; j++)
+					piano_note_off(PIANO(chst->w_piano), j);
+			}
 		}
 		if (is_redirect(port))
 			port_flush_event(port->port);
