@@ -1,7 +1,7 @@
 /*
  * statusbar.c
  *
- * Copyright (c) 1999 by Takashi Iwai <iwai@ww.uni-erlangen.de>
+ * Copyright (c) 1999 by Takashi Iwai <tiwai@suse.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -338,7 +338,7 @@ draw_arrow(GtkWidget *w, status_bar_t *arg)
 			   w->style->black_gc,
 			   TRUE,
 			   0, 0, arg->width, arg->height);
-	p[0].x = arg->drawn;
+	p[0].x = arg->drawn+1;
 	p[0].y = arg->height / 2;
 	p[1].x = arg->width - arg->drawn - 1;
 	p[1].y = 0;
